# -*- coding: utf8 -*-
from __future__ import unicode_literals

import pytest
import six

import driverng
from driverng.protocol.message_pb2 import MCAMessage


def test_legacy_config():
    msg = MCAMessage()
    msg.type = MCAMessage.CONFIG_REPLY
    msg.config.version = "15.9.3"
    for pk, options in enumerate(
        [
            {
                "REPERTOIRE_FTP_ADVIA2120": "mca/2120metz",
                "baud": "9600",
                "encoding": "ISO-8859-15",
                "format_automate": "advia",
                "label": "c_2120_me",
                "n_bit": 8,
                "nom_port": "ttyS0",
                "parite": False,
                "stop_bit": 1,
                "type_serv": 0,
                "xon_in": False,
            },
            {
                "encoding": "ISO-8859-15",
                "fic_in": "/tmp/{count}.TXT",
                "fic_out": "",
                "format_automate": "qwalys",
                "label": "qw",
                "nom_port": "",
                "type_serv": 5,
            },
            {
                "encoding": "ISO-8859-15",
                "format_automate": "ctlink",
                "label": "ctlink1",
                "nom_port": "10.57.10.248:9341",
                "parite": False,
                "type_serv": 6,
            },
            {
                "baud": 9600,
                "encoding": "ISO-8859-15",
                "format_automate": "hemofax",
                "label": "hemofax",
                "mode_query": "true",
                "n_bit": 7,
                "nom_port": "ttyS0",
                "parite": True,
                "stop_bit": 1,
                "type_serv": 0,
                "xon_in": False,
            },
            {
                "encoding": "UTF-8",
                "format_automate": "advia1200",
                "label": "c_ad1200",
                "nom_port": "0.0.0.0:9341",
                "parite": True,
                "type_serv": 6,
            },
            {
                "encoding": "ISO-8859-15",
                "fic_in": "/tmp/Import.TXT",
                "fic_out": "",
                "format_automate": "cla1",
                "label": "cla1",
                "nom_port": "",
                "type_serv": 5,
            },
        ]
    ):
        c = msg.config.config.add()
        c.id = pk
        for key, value in options.items():
            opt = c.options.add()
            opt.key = key
            opt.value = six.text_type(value)

    config = driverng.Config.from_legacy(msg)
    config.set_defaults()
    config.validate()
    expected = {
        "connections": [
            {
                "name": "c_2120_me",
                "id": 0,
                "format": {"type": "advia"},
                "protocol": {
                    "type": "advia2120",
                    "encoding": "ISO-8859-15",
                    "repertoire_ftp": "mca/2120metz",
                },
                "transport": {
                    "type": "serial",
                    "baudrate": 9600,
                    "bytesize": 8,
                    "port": "/dev/ttyS0",
                    "stopbits": 1,
                    "xonxoff": False,
                },
            },
            {
                "name": "qw",
                "id": 1,
                "format": {"type": "qwalys"},
                "protocol": {
                    "type": "void",
                    "multipart_astm": True,
                    "encoding": "ISO-8859-15",
                },
                "transport": {
                    "type": "file",
                    "output_dir": "/tmp",
                    "input_dir": "",
                    "glob": "",
                    "fmt": "{count}.TXT",
                },
            },
            {
                "name": "ctlink1",
                "id": 2,
                "format": {"type": "ctlink"},
                "protocol": {"type": "astm", "encoding": "ISO-8859-15"},
                "transport": {
                    "type": "tcp",
                    "host": "10.57.10.248",
                    "port": 9341,
                    "mode": "client",
                },
            },
            {
                "name": "hemofax",
                "id": 3,
                "format": {"type": "hemofax"},
                "protocol": {
                    "encoding": "ISO-8859-15",
                    "type": "astm",
                    "mode_query": True,
                },
                "transport": {
                    "type": "serial",
                    "baudrate": 9600,
                    "bytesize": 7,
                    "parity": "O",
                    "port": "/dev/ttyS0",
                    "stopbits": 1,
                    "xonxoff": False,
                },
            },
            {
                "name": "c_ad1200",
                "id": 4,
                "format": {"type": "advia1200"},
                "protocol": {
                    "type": "astm",
                    "encoding": "UTF-8",
                    "end_payload_with_cr": False,
                    "max_car_bloc": None,
                },
                "transport": {
                    "type": "tcp",
                    "host": "0.0.0.0",
                    "port": 9341,
                    "mode": "server",
                },
            },
            {
                "name": "cla1",
                "id": 5,
                "format": {"type": "cla1"},
                "protocol": {"type": "void", "encoding": "ISO-8859-15"},
                "transport": {
                    "type": "file",
                    "output_dir": "/tmp",
                    "input_dir": "",
                    "glob": "",
                    "fmt": "Import.TXT",
                    "append": True,
                },
            },
        ]
    }
    assert config == expected


def test_default_config():
    config = driverng.Config(
        {
            "connections": [
                {
                    "name": "C",
                    "format": {"type": "advia1200"},
                    "transport": {
                        "type": "tcp",
                        "mode": "server",
                        "host": "0.0.0.0",
                        "port": 4242,
                    },
                }
            ]
        }
    )
    config.set_defaults()
    config.validate()
    expected = {
        "connections": [
            {
                "name": "C",
                "format": {"type": "advia1200"},
                "protocol": {
                    "type": "astm",
                    "end_payload_with_cr": False,
                    "max_car_bloc": None,
                    "encoding": "ISO-8859-15",
                },
                "transport": {
                    "type": "tcp",
                    "mode": "server",
                    "host": "0.0.0.0",
                    "port": 4242,
                },
            }
        ]
    }
    assert config == expected


def test_invalid_config_protocol():
    config = driverng.Config(
        {
            "connections": [
                {
                    "name": "C",
                    "protocol": {"type": "astm"},
                    "transport": {"type": "file"},
                }
            ]
        }
    )
    config.set_defaults()
    with pytest.raises(driverng.ConfigError) as excinfo:
        config.validate()
    assert "Cannot use protocol astm with transport file" in str(excinfo.value)
